"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BarRenderer = void 0;
const datagrid_1 = require("@lumino/datagrid");
/**
 * A cell renderer which renders data values as bars.
 */
class BarRenderer extends datagrid_1.TextRenderer {
    /**
     * Construct a new bar renderer.
     *
     * @param options - The options for initializing the renderer.
     */
    constructor(options = {}) {
        super(options);
        this.barColor = options.barColor || '#4682b4';
        this.barValue = options.barValue || 0;
        this.orientation = options.orientation || 'horizontal';
        this.barVerticalAlignment = options.barVerticalAlignment || 'bottom';
        this.barHorizontalAlignment = options.barHorizontalAlignment || 'left';
        this.showText = options.showText || true;
    }
    /**
     * Paint the content for a cell.
     *
     * @param gc - The graphics context to use for drawing.
     *
     * @param config - The configuration data for the cell.
     */
    paint(gc, config) {
        const showText = datagrid_1.CellRenderer.resolveOption(this.showText, config);
        this.drawBackground(gc, config);
        this.drawBar(gc, config);
        if (showText)
            this.drawText(gc, config);
    }
    /**
     * Prepare the graphics context for drawing a column of cells.
     *
     * @param gc - The graphics context to prepare.
     *
     * @param row - The index of the first row to be rendered.
     *
     * @param col - The index of the column to be rendered.
     *
     * @param field - The field descriptor for the column, or `null`.
     */
    prepare(gc, config) {
        // Look up the default state from the renderer.
        const { font, textColor, barColor, backgroundColor, horizontalAlignment } = this;
        // Set up the default font.
        if (font && typeof font === 'string') {
            gc.font = font;
        }
        // Set up the default fill style.
        if (backgroundColor && typeof backgroundColor === 'string') {
            gc.fillStyle = backgroundColor;
        }
        else if (barColor && typeof barColor === 'string') {
            gc.fillStyle = barColor;
        }
        else if (textColor && typeof textColor === 'string') {
            gc.fillStyle = textColor;
        }
        // Set up the default text alignment.
        if (typeof horizontalAlignment === 'string') {
            gc.textAlign = horizontalAlignment;
        }
        else {
            gc.textAlign = 'left';
        }
        // Set up the default text baseline.
        gc.textBaseline = 'bottom';
    }
    /**
     * Draw the bar.
     *
     * @param gc - The graphics context to use for drawing.
     *
     * @param config - The configuration data for the cell.
     */
    drawBar(gc, config) {
        const barColor = datagrid_1.CellRenderer.resolveOption(this.barColor, config);
        let barValue = datagrid_1.CellRenderer.resolveOption(this.barValue, config);
        const vAlign = datagrid_1.CellRenderer.resolveOption(this.barVerticalAlignment, config);
        const hAlign = datagrid_1.CellRenderer.resolveOption(this.barHorizontalAlignment, config);
        const orientation = datagrid_1.CellRenderer.resolveOption(this.orientation, config);
        // Bail if there is no color to draw.
        if (!barColor) {
            return;
        }
        // Be careful not to draw outside of the cell
        if (barValue > 1) {
            barValue = 1;
        }
        if (barValue < 0) {
            barValue = 0;
        }
        let x = config.x;
        let y = config.y;
        // Draw the bar in the cell.
        gc.fillStyle = barColor;
        if (orientation === 'horizontal') {
            const rect_width = barValue * config.width;
            if (hAlign === 'center')
                x += (config.width - rect_width) / 2;
            if (hAlign === 'right')
                x += config.width - rect_width;
            gc.fillRect(x, y, rect_width, config.height);
        }
        else {
            const rect_height = barValue * config.height;
            if (vAlign === 'center') {
                y += (config.height - rect_height) / 2;
            }
            else if (vAlign === 'bottom') {
                y += config.height - rect_height;
            }
            gc.fillRect(x, y, config.width, rect_height);
        }
    }
}
exports.BarRenderer = BarRenderer;
//# sourceMappingURL=barrenderer.js.map