"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FeatherGridContextMenu = exports.GridContextMenu = void 0;
const widgets_1 = require("@lumino/widgets");
/**
 * An object which provides context menus for the data grid.
 *
 * #### Notes
 * This item creates one Menu widget, then changes it's items based on
 * the `cellClick` signal from `DataGrid`.
 */
class GridContextMenu {
    /**
     * Construct a new grid context menu.
     *
     * @param options - The options for initializing the context menu.
     */
    constructor(options) {
        this._menu = new widgets_1.Menu({ commands: options.commands });
        this._menu.addClass('ipydatagrid-context-menu');
    }
}
exports.GridContextMenu = GridContextMenu;
/**
 * An WIP object which provides context menus for the data grid.
 *
 * #### Notes
 * This is primarily here for demo purposes to demonstrate how we may want to
 * manage context menus.
 */
class FeatherGridContextMenu extends GridContextMenu {
    /**
     * Opens the context menu in response to the `cellClick` signal of the
     * data grid.
     *
     * @param grid - The "sender" of the signal.
     *
     * @param hit  - The "value" of the signal.
     */
    open(grid, hit) {
        // Discard the current menu items.
        this._menu.clearItems();
        // Create the args that will be provided to the commands' .execute() method
        const args = {
            region: hit.region,
            rowIndex: hit.row,
            columnIndex: hit.column,
            clientX: hit.x,
            clientY: hit.y,
        };
        // Add menu items based on the region of the grid that was clicked on.
        switch (hit.region) {
            case 'column-header':
                this._menu.addItem({
                    command: FeatherGridContextMenu.CommandID.SortAscending,
                    args: args,
                });
                this._menu.addItem({
                    command: FeatherGridContextMenu.CommandID.SortDescending,
                    args: args,
                });
                this._menu.addItem({
                    command: FeatherGridContextMenu.CommandID.SortClear,
                    args: args,
                });
                this._menu.addItem({
                    type: 'separator',
                });
                this._menu.addItem({
                    command: FeatherGridContextMenu.CommandID.OpenFilterByConditionDialog,
                    args: args,
                });
                this._menu.addItem({
                    command: FeatherGridContextMenu.CommandID.OpenFilterByValueDialog,
                    args: args,
                });
                this._menu.addItem({
                    type: 'separator',
                });
                this._menu.addItem({
                    command: FeatherGridContextMenu.CommandID.ClearSelection,
                    args: args,
                });
                this._menu.addItem({
                    command: FeatherGridContextMenu.CommandID.ClearThisFilter,
                    args: args,
                });
                this._menu.addItem({
                    command: FeatherGridContextMenu.CommandID.ClearFiltersInAllColumns,
                    args: args,
                });
                this._menu.addItem({
                    command: FeatherGridContextMenu.CommandID.CopyToClipboard,
                    args: args,
                });
                break;
            case 'corner-header':
                this._menu.addItem({
                    command: FeatherGridContextMenu.CommandID.SortAscending,
                    args: args,
                });
                this._menu.addItem({
                    command: FeatherGridContextMenu.CommandID.SortDescending,
                    args: args,
                });
                this._menu.addItem({
                    command: FeatherGridContextMenu.CommandID.SortClear,
                    args: args,
                });
                this._menu.addItem({
                    type: 'separator',
                });
                this._menu.addItem({
                    command: FeatherGridContextMenu.CommandID.OpenFilterByConditionDialog,
                    args: args,
                });
                this._menu.addItem({
                    command: FeatherGridContextMenu.CommandID.OpenFilterByValueDialog,
                    args: args,
                });
                this._menu.addItem({
                    type: 'separator',
                });
                this._menu.addItem({
                    command: FeatherGridContextMenu.CommandID.CopySelectionToClipboard,
                    args: args,
                });
                this._menu.addItem({
                    command: FeatherGridContextMenu.CommandID.SaveSelectionAsCsv,
                    args: args,
                });
                this._menu.addItem({
                    command: FeatherGridContextMenu.CommandID.SaveAllAsCsv,
                    args: args,
                });
                this._menu.addItem({
                    type: 'separator',
                });
                this._menu.addItem({
                    command: FeatherGridContextMenu.CommandID.ClearSelection,
                    args: args,
                });
                this._menu.addItem({
                    command: FeatherGridContextMenu.CommandID.ClearThisFilter,
                    args: args,
                });
                this._menu.addItem({
                    command: FeatherGridContextMenu.CommandID.ClearFiltersInAllColumns,
                    args: args,
                });
                break;
            case 'body':
                this._menu.addItem({
                    command: FeatherGridContextMenu.CommandID.RevertGrid,
                    args: args,
                });
                break;
            default:
                throw 'unreachable';
        }
        // Open context menu at location of the click event
        this._menu.open(hit.x, hit.y);
        // Issue 422: menu should be first child of document.body not last child to work on all of
        // Jupyter Lab, Notebook < 7, NbClassic and Voila. Until this is available in lumino/widgets,
        // detach and reattach the menu here.
        const bodyFirstChild = document.body.firstElementChild;
        if (this._menu.node.parentElement == document.body &&
            bodyFirstChild != this._menu.node) {
            widgets_1.Widget.detach(this._menu);
            widgets_1.Widget.attach(this._menu, document.body, bodyFirstChild);
        }
    }
}
exports.FeatherGridContextMenu = FeatherGridContextMenu;
/**
 * The namespace for the `IPyDataGridContextMenu` class statics.
 */
(function (FeatherGridContextMenu) {
    /**
     * Command ID strings for the IpyDataGridContextMenu.
     */
    let CommandID;
    (function (CommandID) {
        CommandID["SortAscending"] = "sort:Asc";
        CommandID["SortDescending"] = "sort:Desc";
        CommandID["SortClear"] = "sort:Clear";
        CommandID["OpenFilterByConditionDialog"] = "filterCondition:openDialog";
        CommandID["OpenFilterByValueDialog"] = "filterValue:openDialog";
        CommandID["RevertGrid"] = "grid:reset";
        CommandID["ClearThisFilter"] = "filter:clearCurrentColumn";
        CommandID["ClearFiltersInAllColumns"] = "filter:clearAllColumns";
        CommandID["CopyToClipboard"] = "copyToClipboard";
        CommandID["CopySelectionToClipboard"] = "copySelectionToClipboard";
        CommandID["SaveSelectionAsCsv"] = "saveSelectionAsCsv";
        CommandID["SaveAllAsCsv"] = "saveAllAsCsv";
        CommandID["ClearSelection"] = "clearSelection";
    })(CommandID = FeatherGridContextMenu.CommandID || (FeatherGridContextMenu.CommandID = {}));
})(FeatherGridContextMenu = exports.FeatherGridContextMenu || (exports.FeatherGridContextMenu = {}));
//# sourceMappingURL=gridContextMenu.js.map