"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.StreamingViewBasedJSONModel = void 0;
const deserialize_1 = require("./deserialize");
const streamingview_1 = require("./streamingview");
const viewbasedjsonmodel_1 = require("./viewbasedjsonmodel");
/**
 * A view based data model implementation for in-memory JSON data.
 */
class StreamingViewBasedJSONModel extends viewbasedjsonmodel_1.ViewBasedJSONModel {
    constructor(options) {
        super(options);
    }
    setModelRangeData(r1, r2, c1, c2, value) {
        this._currentView.setDataRange(r1, r2, c1, c2, value);
        this.emitChanged({
            type: 'cells-changed',
            region: 'body',
            row: r1,
            column: c1,
            rowSpan: r2 - r1 + 1,
            columnSpan: c2 - c1 + 1,
        });
        this.emitChanged({
            type: 'cells-changed',
            region: 'row-header',
            row: r1,
            column: 0,
            rowSpan: r2 - r1 + 1,
            columnSpan: this._currentView.columnCount('row-header'),
        });
    }
    /**
     * Returns a Promise that resolves to an array of unique values contained in
     * the provided column index.
     *
     * @param region - The CellRegion to retrieve unique values for.
     * @param column - The column to retrieve unique values for.
     */
    uniqueValues(region, column) {
        if (this._unique &&
            region == this._unique.region &&
            column == this._unique.column) {
            return Promise.resolve(this._unique.values);
        }
        const promise = new Promise(resolve => {
            this._dataModel.once('msg:custom', (content, buffers) => {
                if (content.event_type === 'unique-values-reply') {
                    const { value } = content;
                    // Bring back buffers at their original position in the data structure
                    for (const col of Object.keys(value.data)) {
                        if (value.data[col].type !== 'raw') {
                            value.data[col].value = buffers[value.data[col].value];
                        }
                    }
                    const deserialized = (0, deserialize_1.deserialize_data_simple)(value, null);
                    const values = deserialized[content.column];
                    this._unique = { region, column: content.column, values };
                    resolve(this._unique.values);
                }
            });
        });
        const msg = { type: 'unique-values-request', column: column };
        this._dataModel.send(msg);
        return promise;
    }
    updateDataset(options) {
        this._dataModel = options.dataModel;
        this._dataset = options.datasource;
        this._updatePrimaryKeyMap();
        const view = new streamingview_1.StreamingView({
            datasource: this._dataset,
            rowCount: options.rowCount,
        });
        this.currentView = view;
    }
    updateCellValue(options) {
        if (options.region != 'body') {
            return;
        }
        this._currentView.setData(options.value, options.row, options.column);
    }
    get currentView() {
        return this._currentView;
    }
    set currentView(view) {
        super.currentView = view;
    }
    /**
     * Handler for transformState.changed events.
     *
     * @param sender - TransformStateManager
     *
     * @param value - Event.
     */
    _transformStateChangedHandler(sender, value) {
        this._transformSignal.emit(value);
    }
}
exports.StreamingViewBasedJSONModel = StreamingViewBasedJSONModel;
//# sourceMappingURL=streamingviewbasedjsonmodel.js.map