import { Transform } from './transform';
import { DataSource } from '../datasource';
/**
 * An object that defines a data transformation executor.
 */
export declare abstract class TransformExecutor {
    /**
     * Apply a transformation to the provided data, then return a new copy.
     */
    abstract apply(input: TransformExecutor.IData): TransformExecutor.IData;
}
/**
 * The namespace for the `Transform` class statics.
 */
export declare namespace TransformExecutor {
    /**
     * A read only type for the input/output of .apply().
     */
    type IData = Readonly<DataSource>;
}
/**
 * A transformation that filters a single field by the provided operator and
 * value.
 *
 * Note: This is a WIP
 */
export declare class FilterExecutor extends TransformExecutor {
    /**
     * Create a new transformation.
     *
     * @param options - The options for initializing the transformation.
     */
    constructor(options: FilterExecutor.IOptions);
    /**
     * Apply a transformation to the provided data.
     *
     * @param input - The data to be operated on.
     */
    apply(input: TransformExecutor.IData): TransformExecutor.IData;
    protected _options: FilterExecutor.IOptions;
}
/**
 * The namespace for the `FilterExecutor` class statics.
 */
export declare namespace FilterExecutor {
    /**
     * An options object for initializing a Filter.
     */
    interface IOptions {
        /**
         * The name of the field in the data source.
         */
        field: string;
        /**
         * The data type of the column associated with this transform.
         */
        dType: string;
        /**
         * The operator to use for the comparison.
         */
        operator: Transform.FilterOperator;
        /**
         * The value(s) to filter by.
         */
        value: string | string[] | number | number[];
    }
}
/**
 * A transformation that sorts the provided data by the provided field.
 *
 * Note: This is a WIP
 */
export declare class SortExecutor extends TransformExecutor {
    /**
     * Creates a new sort transformation
     *
     * @param options - The options for initializing the transformation.
     */
    constructor(options: SortExecutor.IOptions);
    /**
     * Apply a transformation to the provided data.
     *
     * Note: Currently ignores the `desc` parameter.
     *
     * @param input - The data to be operated on.
     */
    apply(input: TransformExecutor.IData): TransformExecutor.IData;
    protected _options: SortExecutor.IOptions;
}
/**
 * The namespace for the `SortExecutor` class statics.
 */
export declare namespace SortExecutor {
    /**
     * An options object for initializing a Sort.
     */
    interface IOptions {
        /**
         * The name of the field in the data source.
         */
        field: string;
        /**
         * The data type of the column associated with this transform.
         */
        dType: string;
        /**
         * Indicates ascending or descending order for the sort.
         */
        desc: boolean;
    }
}
export declare namespace Private {
    type JSONDate = 'date' | 'time' | 'datetime';
}
