import { Transform } from './transform';
import { View } from './view';
import { ISignal } from '@lumino/signaling';
import { DataSource } from '../datasource';
/**
 * A state manager for tracking the active transformations of a model.
 */
export declare class TransformStateManager {
    protected _add(transform: Transform.TransformSpec): void;
    /**
     * Adds the provided transform to the active state.
     *
     * @param transform - The transform to be added.
     */
    add(transform: Transform.TransformSpec): void;
    /**
     * Replaces the existing state with the provided list of transforms.
     *
     * @param transforms - The list of transforms to be added.
     */
    replace(transforms: Transform.TransformSpec[]): void;
    /**
     * Creates a new data model View with the active transformations.
     *
     * @param data - The dataset to operate on.
     */
    createView(data: Readonly<DataSource>): View;
    /**
     * Creates an optimized list of TransformExecutors for the provided data.
     *
     * @param data - The dataset to operate on.
     */
    private _createExecutors;
    /**
     * Removes the provided transformation from the active state.
     *
     * @param column - The index of the column state to be removed.
     *
     * @param transformType - The type of the transform to be removed from state.
     */
    remove(column: string, transformType: string): void;
    /**
     * Returns the transform metadata for the provided column.
     *
     * @param column - The column index of the metadata to be retrieved.
     */
    metadata(column: string): TransformStateManager.IColumn | undefined;
    /**
     * Removes all transformations from the active state.
     */
    clear(): void;
    /**
     * A signal emitted when the transform state has changed.
     */
    get changed(): ISignal<this, TransformStateManager.IEvent>;
    /**
     * Returns an array of the active Transforms in state
     */
    get activeTransforms(): Transform.TransformSpec[];
    getFilterTransform(column: string): Transform.TransformSpec | undefined;
    private _state;
    private _changed;
}
/**
 * The namespace for the `TransformStateManager` class statics.
 */
export declare namespace TransformStateManager {
    /**
     * An object when specifies the schema for a single column of state.
     */
    interface IColumn {
        filter: Transform.Filter | undefined;
        sort: Transform.Sort | undefined;
    }
    interface IState {
        [key: string]: IColumn;
    }
    interface IEvent {
        readonly type: 'transforms-updated';
        readonly transforms: Transform.TransformSpec[];
    }
}
