"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TransformStateManager = void 0;
const view_1 = require("./view");
const transformExecutors_1 = require("./transformExecutors");
const algorithm_1 = require("@lumino/algorithm");
const coreutils_1 = require("@lumino/coreutils");
const signaling_1 = require("@lumino/signaling");
/**
 * A state manager for tracking the active transformations of a model.
 */
class TransformStateManager {
    constructor() {
        this._state = {};
        this._changed = new signaling_1.Signal(this);
    }
    _add(transform) {
        // Add column to state if not already present
        if (!this._state.hasOwnProperty(transform.column)) {
            this._state[transform.column] = {
                sort: undefined,
                filter: undefined,
            };
        }
        // Add the transform to the state
        switch (transform.type) {
            case 'sort':
                // Only allow one sort transform.
                // TODO: Support multiple sort columns.
                for (const key of Object.keys(this._state)) {
                    this._state[key]['sort'] = undefined;
                }
                this._state[transform.column]['sort'] = transform;
                break;
            case 'filter':
                this._state[transform.column]['filter'] = transform;
                break;
            default:
                throw 'unreachable';
        }
    }
    /**
     * Adds the provided transform to the active state.
     *
     * @param transform - The transform to be added.
     */
    add(transform) {
        try {
            this._add(transform);
            this._changed.emit({
                type: 'transforms-updated',
                transforms: this.activeTransforms,
            });
        }
        catch (err) {
            this.clear();
        }
    }
    /**
     * Replaces the existing state with the provided list of transforms.
     *
     * @param transforms - The list of transforms to be added.
     */
    replace(transforms) {
        // Bail if the transforms are the same as the current state
        if (coreutils_1.JSONExt.deepEqual(transforms, this.activeTransforms)) {
            return;
        }
        try {
            this._state = {};
            (0, algorithm_1.each)(transforms, (transform) => {
                this._add(transform);
            });
            this._changed.emit({
                type: 'transforms-updated',
                transforms: this.activeTransforms,
            });
        }
        catch (err) {
            this.clear();
        }
    }
    /**
     * Creates a new data model View with the active transformations.
     *
     * @param data - The dataset to operate on.
     */
    createView(data) {
        const executors = this._createExecutors(data);
        let transformedData = data;
        (0, algorithm_1.each)(executors, (transform) => {
            transformedData = transform.apply(transformedData);
        });
        return new view_1.View(transformedData);
    }
    /**
     * Creates an optimized list of TransformExecutors for the provided data.
     *
     * @param data - The dataset to operate on.
     */
    _createExecutors(data) {
        const sortExecutors = [];
        const filterExecutors = [];
        Object.keys(this._state).forEach((column) => {
            const transform = this._state[column];
            if (transform.sort) {
                let dType = '';
                for (const field of data.schema.fields) {
                    if (field.name === transform.sort.column) {
                        dType = field.type;
                    }
                }
                const executor = new transformExecutors_1.SortExecutor({
                    field: transform.sort.column,
                    dType,
                    desc: transform.sort.desc,
                });
                sortExecutors.push(executor);
            }
            if (transform.filter) {
                let dType = '';
                for (const field of data.schema.fields) {
                    if (field.name === transform.filter.column) {
                        dType = field.type;
                    }
                }
                const executor = new transformExecutors_1.FilterExecutor({
                    field: transform.filter.column,
                    dType,
                    operator: transform.filter.operator,
                    value: transform.filter.value,
                });
                filterExecutors.push(executor);
            }
        });
        // Always put filters first
        return [...filterExecutors, ...sortExecutors];
    }
    /**
     * Removes the provided transformation from the active state.
     *
     * @param column - The index of the column state to be removed.
     *
     * @param transformType - The type of the transform to be removed from state.
     */
    remove(column, transformType) {
        // Return immediately if the key is not in the state
        if (!this._state.hasOwnProperty(column)) {
            return;
        }
        try {
            const columnState = this._state[column];
            if (transformType === 'sort') {
                columnState.sort = undefined;
            }
            else if (transformType === 'filter') {
                columnState.filter = undefined;
            }
            else {
                throw 'unreachable';
            }
            if (!columnState.sort && !columnState.filter) {
                delete this._state[column];
            }
            this._changed.emit({
                type: 'transforms-updated',
                transforms: this.activeTransforms,
            });
        }
        catch (err) {
            this.clear();
        }
    }
    /**
     * Returns the transform metadata for the provided column.
     *
     * @param column - The column index of the metadata to be retrieved.
     */
    metadata(column) {
        if (!this._state.hasOwnProperty(column)) {
            return undefined;
        }
        else {
            return this._state[column];
        }
    }
    /**
     * Removes all transformations from the active state.
     */
    clear() {
        this._state = {};
        this._changed.emit({
            type: 'transforms-updated',
            transforms: this.activeTransforms,
        });
    }
    /**
     * A signal emitted when the transform state has changed.
     */
    get changed() {
        return this._changed;
    }
    /**
     * Returns an array of the active Transforms in state
     */
    get activeTransforms() {
        const transforms = [];
        (0, algorithm_1.each)(Object.keys(this._state), (column) => {
            if (this._state[column].sort) {
                transforms.push(this._state[column].sort);
            }
            if (this._state[column].filter) {
                transforms.push(this._state[column].filter);
            }
        });
        return transforms;
    }
    getFilterTransform(column) {
        if (!this._state.hasOwnProperty(column)) {
            return undefined;
        }
        return this._state[column].filter;
    }
}
exports.TransformStateManager = TransformStateManager;
//# sourceMappingURL=transformStateManager.js.map