"use strict";
/*-----------------------------------------------------------------------------
| Copyright (c) 2014-2017, PhosphorJS Contributors
|
| Distributed under the terms of the BSD 3-Clause License.
|
| The full license is in the file LICENSE, distributed with this software.
|----------------------------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
exports.View = void 0;
/**
 * A View implementation for immutable in-memory JSON data.
 *
 * Note: Most of this is just repurposed from JSONModel, and can likely be
 * streamlined quite a bit.
 */
class View {
    /**
     * Create a view with static JSON data.
     *
     * @param datasource - The datasource for initializing the view.
     */
    constructor(datasource) {
        const split = Private.splitFields(datasource.schema);
        this._data = datasource;
        this._bodyFields = split.bodyFields;
        this._headerFields = split.headerFields;
        this._missingValues = Private.createMissingMap(datasource.schema);
    }
    /**
     * Get the row count for a region in the view.
     *
     * @param region - The row region of interest.
     *
     * @returns - The row count for the region.
     */
    rowCount(region) {
        var _a, _b;
        if (region === 'body') {
            return this._data.length;
        }
        else {
            return (_b = (_a = this._bodyFields[0]) === null || _a === void 0 ? void 0 : _a.rows.length) !== null && _b !== void 0 ? _b : 1;
        }
    }
    /**
     * Get the column count for a region in the view.
     *
     * @param region - The column region of interest.
     *
     * @returns - The column count for the region.
     */
    columnCount(region) {
        if (region === 'body') {
            return this._bodyFields.length;
        }
        return this._headerFields.length;
    }
    /**
     * Get the metadata for a column in the view.
     *
     * @param region - The cell region of interest.
     *
     * @param column - The index of the column of interest.
     *
     * @returns The metadata for the column.
     */
    metadata(region, row, column) {
        if (region === 'body' || region === 'column-header') {
            return Object.assign({ row: row, column: column }, this._bodyFields[column]);
        }
        return Object.assign({ row: row, column: column }, this._headerFields[column]);
    }
    /**
     * Get the data value for a cell in the view.
     *
     * @param region - The cell region of interest.
     *
     * @param row - The row index of the cell of interest.
     *
     * @param column - The column index of the cell of interest.
     *
     * @param returns - The data value for the specified cell.
     *
     * #### Notes
     * A `missingValue` as defined by the schema is converted to `null`.
     */
    data(region, row, column) {
        // Set up the field and value variables.
        let field;
        let value;
        // Look up the field and value for the region.
        switch (region) {
            case 'body':
                field = this._bodyFields[column];
                value = this._data.data[field.name][row];
                break;
            case 'column-header':
                field = this._bodyFields[column];
                value = field.rows[row];
                break;
            case 'row-header':
                field = this._headerFields[column];
                value = this._data.data[field.name][row];
                break;
            case 'corner-header':
                field = this._headerFields[column];
                value = field.rows[row];
                break;
            default:
                throw 'unreachable';
        }
        // Test whether the value is a missing value.
        const missing = this._missingValues !== null &&
            typeof value === 'string' &&
            this._missingValues[value] === true;
        // Return the final value.
        return missing ? null : value;
    }
    /**
     * Returns a reference to the dataset from this View.
     */
    get dataset() {
        return this._data;
    }
    /**
     * Returns the index in the schema that relates to the index by region.
     *
     * @param region - The `CellRegion` of interest.
     *
     * @param index - The column index to look up.
     */
    getSchemaIndex(region, index) {
        if (region === 'corner-header') {
            return index;
        }
        else {
            return this._headerFields.length + index;
        }
    }
    /**
     * Returns a Promise that resolves to an array of unique values contained in
     * the provided column index.
     *
     * @param column - The column to retrieve unique values for.
     */
    uniqueValues(region, column) {
        return Array.from(new Set(this.dataset.data[column]));
    }
}
exports.View = View;
/**
 * The namespace for the module implementation details.
 */
var Private;
(function (Private) {
    /**
     * Split the schema fields into header and body fields.
     */
    function splitFields(schema) {
        // Normalize the primary keys.
        let primaryKeys;
        if (schema.primaryKey === undefined) {
            primaryKeys = [];
        }
        else if (typeof schema.primaryKey === 'string') {
            primaryKeys = [schema.primaryKey];
        }
        else {
            primaryKeys = schema.primaryKey;
        }
        // Separate the fields for the body and header.
        const bodyFields = [];
        const headerFields = [];
        for (const field of schema.fields) {
            // Skipping the primary key unique identifier so
            // it is not rendered.
            if (field.rows[0] == schema.primaryKeyUuid) {
                continue;
            }
            if (primaryKeys.indexOf(field.name) === -1) {
                bodyFields.push(field);
            }
            else {
                headerFields.push(field);
            }
        }
        // Return the separated fields.
        return { bodyFields, headerFields };
    }
    Private.splitFields = splitFields;
    /**
     * Create a missing values map for a schema.
     *
     * This returns `null` if there are no missing values.
     */
    function createMissingMap(schema) {
        // Bail early if there are no missing values.
        if (!schema.missingValues || schema.missingValues.length === 0) {
            return null;
        }
        // Collect the missing values into a map.
        const result = Object.create(null);
        for (const value of schema.missingValues) {
            result[value] = true;
        }
        // Return the populated map.
        return result;
    }
    Private.createMissingMap = createMissingMap;
})(Private || (Private = {}));
//# sourceMappingURL=view.js.map