"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MouseHandler = void 0;
const datagrid_1 = require("@lumino/datagrid");
const domutils_1 = require("@lumino/domutils");
const signaling_1 = require("@lumino/signaling");
const headerRenderer_1 = require("./core/headerRenderer");
class MouseHandler extends datagrid_1.BasicMouseHandler {
    /**
     * Construct a new datagrid mouse handler.
     *
     * @param grid - The FeatherGrid object for which mouse events are handled.
     */
    constructor(grid) {
        super();
        this._mouseIsDown = false;
        this._cellClicked = new signaling_1.Signal(this);
        this._grid = grid;
    }
    /**
     * Handle the mouse down event for the data grid.
     *
     * @param grid - The data grid of interest.
     *
     * @param event - The mouse down event of interest.
     */
    onMouseDown(grid, event) {
        const hit = grid.hitTest(event.clientX, event.clientY);
        const hitRegion = hit.region;
        const buttonSize = headerRenderer_1.HeaderRenderer.iconWidth * 1.5;
        const buttonPadding = headerRenderer_1.HeaderRenderer.buttonPadding;
        const accel = domutils_1.Platform.accelKey(event);
        this._mouseIsDown = true;
        // Send cell clicked signal
        if (hit.region !== 'void') {
            this._cellClicked.emit(hit);
        }
        if (hitRegion === 'corner-header' || hitRegion === 'column-header') {
            const columnWidth = grid.columnSize(hitRegion === 'corner-header' ? 'row-header' : 'body', hit.column);
            const rowHeight = grid.rowSize('column-header', hit.row);
            const isMenuRow = (hit.region === 'column-header' &&
                hit.row ==
                    this._grid.grid.dataModel.rowCount('column-header') - 1) ||
                (hit.region === 'corner-header' && hit.row === 0);
            const isMenuClick = hit.x > columnWidth - buttonSize - buttonPadding &&
                hit.x < columnWidth - buttonPadding &&
                hit.y > rowHeight - buttonSize - buttonPadding &&
                hit.y < rowHeight - buttonPadding &&
                isMenuRow;
            if (isMenuClick) {
                this._grid.contextMenu.open(grid, Object.assign(Object.assign({}, hit), { x: event.clientX + window.scrollX, y: event.clientY + window.scrollY }));
                return;
            }
        }
        if (grid) {
            // Create cell config object.
            const config = MouseHandler.createCellConfigObject(grid, hit);
            // Bail if no cell config object is defined for the region.
            if (!config) {
                return;
            }
            // Retrieve cell renderer.
            const renderer = grid.cellRenderers.get(config);
            // Only process hyperlink renderers.
            if (renderer instanceof datagrid_1.HyperlinkRenderer) {
                // Use the url param if it exists.
                let url = datagrid_1.CellRenderer.resolveOption(renderer.url, config);
                // Otherwise assume cell value is the URL.
                if (!url) {
                    const format = datagrid_1.TextRenderer.formatGeneric();
                    url = format(config);
                }
                // Emit message to open the hyperlink only if user hit Ctrl+Click.
                if (accel) {
                    // Emit event that will be caught in case window.open is blocked
                    window.postMessage({
                        id: 'ipydatagrid::hyperlinkclick',
                        url,
                    }, '*');
                    // Reset cursor default after clicking
                    const cursor = this.cursorForHandle('none');
                    grid.viewport.node.style.cursor = cursor;
                }
            }
        }
        super.onMouseDown(grid, event);
    }
    onMouseUp(grid, event) {
        this._mouseIsDown = false;
        super.onMouseUp(grid, event);
    }
    get mouseIsDown() {
        return this._mouseIsDown;
    }
    get isResizing() {
        return (this.pressData !== null &&
            (this.pressData.type == 'column-resize' ||
                this.pressData.type == 'row-resize'));
    }
    /**
     * A signal emitted when a grid cell is clicked.
     */
    get cellClicked() {
        return this._cellClicked;
    }
    /**
     * Creates a CellConfig object from a hit region.
     */
    static createCellConfigObject(grid, hit) {
        const { region, row, column } = hit;
        // Terminate call if region is void.
        if (region === 'void') {
            return undefined;
        }
        // Augment hit region params with value and metadata.
        const value = grid.dataModel.data(region, row, column);
        const metadata = grid.dataModel.metadata(region, row, column);
        // Create cell config object to retrieve cell renderer.
        const config = Object.assign(Object.assign({}, hit), { value: value, metadata: metadata });
        return config;
    }
}
exports.MouseHandler = MouseHandler;
//# sourceMappingURL=mousehandler.js.map